
//

const Cc = Components.classes;
const Ci = Components.interfaces;

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

const SCENARIPROTOCOL_SCHEME = "scenari";
const PROTOCOL_HANDLER_OVERLAY_URI = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).newURI("chrome://scenariBootApp/content/scenariProtocolHandler.jsm", null, null);

const scenari_scAppFileName         = "4.scenari";
const scenari_mainWindowType        = "scenari:wsp";

var scenariProtocol = function () { }

scenariProtocol.prototype = {
		classDescription : "scenariProtocol",
		classID : Components.ID("{03E944ED-AC69-41C0-9945-0662A0863962}"),
		contractID : "@mozilla.org/network/protocol;1?name=" + SCENARIPROTOCOL_SCHEME,
		QueryInterface : XPCOMUtils.generateQI([Ci.nsIProtocolHandler]),

		defaultPort: -1,
		scheme: SCENARIPROTOCOL_SCHEME,
		protocolFlags: Ci.nsIProtocolHandler.URI_LOADABLE_BY_ANYONE | Ci.nsIProtocolHandler.URI_NOAUTH | Ci.nsIProtocolHandler.URI_NON_PERSISTABLE | Ci.nsIProtocolHandler.URI_NORELATIVE,

		allowPort: function(pPort, pScheme) {
			return false;
		},

		newURI: function(pSpec, pCharset, pBaseURI) {
			return new ScenariUri(pSpec);
		},

		newChannel: function(pUri) {
			try { 
				var vInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
				var vIsFfXpi = vInfo.ID == "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
				
				var vSpecificPart = pUri.spec.substring(pUri.scheme.length + 1);
				
				var vParams = JSON.parse(vSpecificPart);
				
				if (!vParams) debug("Incorrect scheme-specific part: "+pUri);
	
				var windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
				var window = windowMediator.getMostRecentWindow(null);
				

				var vMustBeReboot = false;
				/* TODO : à revoir lors de la réintégration de sc dans Ff
				if(vParams.extensionId && vIsFfXpi){
			        var vExtensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(Ci.nsIExtensionManager);
			        var vExtensionDs = vExtensionManager.datasource;
			        var vRdfs = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
			        var vContainer = Cc["@mozilla.org/rdf/container;1"].createInstance(Ci.nsIRDFContainer);
			        vContainer.Init(vExtensionManager.datasource, vRdfs.GetResource("urn:mozilla:item:root"));
			        var vXpiFound = false;
			        vElements = vContainer.GetElements();
			        while(vElements.hasMoreElements()){
			            var vElement = vElements.getNext().QueryInterface(Ci.nsIRDFResource);
			            var vType = vExtensionDs.GetTarget(vElement, vRdfs.GetResource("http://www.mozilla.org/2004/em-rdf#type"), true);
			            if (vType && vType instanceof Ci.nsIRDFInt) {
			                if(vType.Value==2){//extension
			                    var vId = vElement.Value;
			                    var vIsDisabled = false;
			                    var vVersion = "0.0.0";
			                    var vIsScExtension = false;
			                    vId = vId.substring("urn:mozilla:item:".length , vId.length);
			                    var vDisabledObj = vExtensionDs.GetTarget(vElement, vRdfs.GetResource("http://www.mozilla.org/2004/em-rdf#userDisabled"), true);
			                    if(vDisabledObj && vDisabledObj instanceof Ci.nsIRDFLiteral){
			                        vIsDisabled = vDisabledObj.QueryInterface(Ci.nsIRDFLiteral).Value=="true" || vDisabledObj.QueryInterface(Ci.nsIRDFLiteral).Value=="needs-enable";
			                    }
			                    var vVersionObj = vExtensionDs.GetTarget(vElement, vRdfs.GetResource("http://www.mozilla.org/2004/em-rdf#version"), true);
			                    if(vVersionObj && vVersionObj instanceof Ci.nsIRDFLiteral){
			                    	vVersion = vVersionObj.QueryInterface(Ci.nsIRDFLiteral).Value;
			                    }

			                    var vInstallLocation = vExtensionManager.getInstallLocation(vId);
			                    var vExtRepository = vInstallLocation.location;
			                    if(vExtRepository){
				                    vExtRepository.append(vId);
				                    if(vExtRepository.exists() && vExtRepository.isDirectory()){
				                        var vLocalExtFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
				                        vLocalExtFile.initWithFile(vExtRepository);
				                        vLocalExtFile.append(scenari_scAppFileName);
				                        if(vLocalExtFile && vLocalExtFile.exists()) vIsScExtension = true;
				                    }
			                    }
			                    if(vIsScExtension){
			                    	if(vId==vParams.extensionId && checkVersion(vVersion, vParams.minVersion, vParams.maxVersion)) {

				                        if(vIsDisabled){
				                        	vExtensionManager.enableItem(vId);
				                        	vMustBeReboot = true;
				                        }
				                        vXpiFound = true;
				                    }else if(!vIsDisabled){

				                    	vExtensionManager.disableItem(vId);
				                        vMustBeReboot = true;
				                    }
			                    }
			                }
			            }
			        }
			        if(!vXpiFound) vMustBeReboot = false;

			        if(vMustBeReboot && confirm("Firefox shall be restarted. Do you want to restart Firefox now\u00A0?", null, "yes", "no")){
			            var vPrefSvc = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);;
			            function appRestart() {
					        Cc["@mozilla.org/toolkit/app-startup;1"].getService(Ci.nsIAppStartup).quit(Ci.nsIAppStartup.eRestart | Ci.nsIAppStartup.eAttemptQuit);
						}
			            window.setTimeout(appRestart, 500);// setTimeOut pour contourner bug sous Linux
			        }
				}
				*/
				
				function checkInstallation(){
					return !vIsFfXpi || (!vMustBeReboot && window.scAppList && window.scAppList[vParams.extensionId] && checkVersion(window.scAppList[vParams.extensionId].version, vParams.minVersion, vParams.maxVersion));
				}
				
				if (vParams.action == 'checkInstallation') {
					var vData = "";
					if (checkInstallation()) vData = "scInstallation = true;"
					return Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).newChannel("data:text/plain," + vData, null, null);
				}else if (vParams.action == 'launch') {
					if(checkInstallation()) window.scAppList[vParams.extensionId].open();
					return Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).newChannel("javascript:", null, null);
				}
				
				var vLastHandler;
				var vLastAuthority = -1
				var vOverlayList = Cc["@mozilla.org/chrome/chrome-registry;1"].getService(Ci.nsIXULOverlayProvider).getXULOverlays(PROTOCOL_HANDLER_OVERLAY_URI);
				while(vOverlayList.hasMoreElements()){
					var vJsmUri = vOverlayList.getNext().QueryInterface(Ci.nsIURI);
					try {

						var vHandler = {};
						Components.utils.import(vJsmUri.spec, vHandler);
						var vNewAuthority = vHandler.getLevelAuthority(window, vParams);
						if(vLastAuthority < vNewAuthority) {
							vLastAuthority = vNewAuthority;
							vLastHandler = vHandler;
						}
					} catch(e) {
						debug("Scenari protocol handler '"+vJsmUri.spec+"' failed : "+e);
					}
				}
				
				if(vLastAuthority>-1) {

					return vLastHandler.newChannel(window, vParams);
				}
				

				debug("Scenari protocol : no handler found for : "+pUri.spec);
				return Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).newChannel("javascript:", null, null);
				
			}catch(e){
				debug("Execute scenari protocol failed ("+pUri.spec+") :: "+e);
				return Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).newChannel("javascript:", null, null);
			}
		}

};

function ScenariUri(pSpec){
	this.fSpec = pSpec || "";
}
ScenariUri.prototype = {
	get asciiHost (){return "";},
	get asciiSpec (){return EncodeUriComponent(this.fSpec);},
	get originCharset (){return "";},
	get prePath (){return "";},
	
	get host (){return "";},
	set host (pValue){return "";},
	
	get hostPort (){return "";},
	set hostPort (pValue){return "";},
	
	get password (){return "";},
	set password (pValue){return "";},
	
	get path (){return "";},
	set path (pValue){return "";},
	
	get port (){return 0;},
	set port (pValue){return 0;},
	
	get scheme (){return SCENARIPROTOCOL_SCHEME;},
	set scheme (pValue){return SCENARIPROTOCOL_SCHEME;},
	
	get spec (){return this.fSpec;},
	set spec (pValue){this.fSpec = pValue; return this.fSpec;},
	
	get userPass (){return "";},
	set userPass (pValue){return "";},
	
	get username (){return "";},
	set username (pValue){return "";},
	
	clone : function(){
		return new ScenariUri(this.fSpec);
	},
	equals : function(pOther){
		if(!pOther || ! pOther.spec) return false;
		return pOther.spec == this.fSpec;
	},
	resolve : function(pRelativePath){
		return new ScenariUri(pRelativePath);
	},
	schemeIs : function(pScheme){
		return pScheme == SCENARIPROTOCOL_SCHEME;
	},
	QueryInterface : XPCOMUtils.generateQI([Ci.nsIURI])
}

function debug(pText){
	var vConsoleService = Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService);
	vConsoleService.logStringMessage("scenariProtocol > " + pText);
}

function alert(pTitle, pText){
	var vService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
	vService.alert(null, pTitle, pText);
	return false;
}

function confirm(pText, pTitle, pLabelOk, pLabelCancel, pCheckMsg, pCheckedObj, pWindow){
	try{
		var vService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
		var vFlag = vService.BUTTON_TITLE_IS_STRING * vService.BUTTON_POS_0 + vService.BUTTON_TITLE_IS_STRING * vService.BUTTON_POS_1;
		
		return vService.confirmEx(	pWindow, 
									pTitle || "Confirmation", 
									pText,
									vFlag,
									pLabelOk || "Ok",
									pLabelCancel || "Cancel",
									null,
									pCheckMsg, 
									pCheckedObj || {}
								  ) == 0;
	} catch(e) {debug("confirm failed:: "+e);}
	return true;
}

function checkVersion(pVersion, pMin, pMax){
	try{
		if(pVersion){
			var vVersionComparator = Cc["@mozilla.org/xpcom/version-comparator;1"].getService(Ci.nsIVersionComparator);
			var vIsCorrect = true;
			if(pMax) vIsCorrect = vVersionComparator.compare(pVersion, pMax)<=0;
			if(vIsCorrect && pMin) vIsCorrect = vVersionComparator.compare(pVersion, pMin)>=0;

			return vIsCorrect;
		}
		return true;
	} catch(e) {debug("checkVersion failed:: "+e);return true;}
}

var NSGetFactory = XPCOMUtils.generateNSGetFactory([scenariProtocol]);

//